@@ -14,6 +14,7 @@
 #include <rtosc/ports.h>
 #include <rtosc/port-sugar.h>
 #include <iostream>
+#include <cassert>
 
 
 #include "EffectMgr.h"
@@ -33,10 +34,44 @@
 
 
 #define rObject EffectMgr
+#define rSubtype(name) \
+    {STRINGIFY(name)"/", NULL, &name::ports,\
+        [](const char *msg, rtosc::RtData &data){\
+            rObject &o = *(rObject*)data.obj; \
+            data.obj = o.efx; \
+            if(!dynamic_cast<name*>(o.efx)) \
+                return; \
+            SNIP \
+            name::ports.dispatch(msg, data); \
+        }}
 static const rtosc::Ports local_ports = {
     rSelf(EffectMgr),
     rPaste,
     rRecurp(filterpars, "Filter Parameter for Dynamic Filter"),
+    {"Pvolume::i", rProp(parameter) rLinear(0,127) rShort("amt") rDoc("amount of effect"),
+        0,
+        [](const char *msg, rtosc::RtData &d)
+        {
+            EffectMgr *eff = (EffectMgr*)d.obj;
+            if(!rtosc_narguments(msg))
+                d.reply(d.loc, "i", eff->geteffectparrt(0));
+            else if(rtosc_type(msg, 0) == 'i'){
+                eff->seteffectparrt(0, rtosc_argument(msg, 0).i);
+                d.broadcast(d.loc, "i", eff->geteffectparrt(0));
+            }
+        }},
+    {"Ppanning::i", rProp(parameter) rLinear(0,127) rShort("pan") rDoc("panning"),
+        0,
+        [](const char *msg, rtosc::RtData &d)
+        {
+            EffectMgr *eff = (EffectMgr*)d.obj;
+            if(!rtosc_narguments(msg))
+                d.reply(d.loc, "i", eff->geteffectparrt(1));
+            else if(rtosc_type(msg, 0) == 'i'){
+                eff->seteffectparrt(1, rtosc_argument(msg, 0).i);
+                d.broadcast(d.loc, "i", eff->geteffectparrt(1));
+            }
+        }},
     {"parameter#128::i:T:F", rProp(parameter) rProp(alias) rLinear(0,127) rDoc("Parameter Accessor"),
         NULL,
         [](const char *msg, rtosc::RtData &d)
@@ -71,7 +106,7 @@ static const rtosc::Ports local_ports = {
 
                 //update parameters as well
                 strncpy(loc, d.loc, 1024);
-                char *tail = rindex(loc, '/');
+                char *tail = strrchr(loc, '/');
                 if(!tail)
                     return;
                 for(int i=0;i<128;++i) {
@@ -94,7 +129,9 @@ static const rtosc::Ports local_ports = {
             eq->getFilter(a,b);
             d.reply(d.loc, "bb", sizeof(a), a, sizeof(b), b);
         }},
-    {"efftype::i", rProp(parameter) rDoc("Get Effect Type"), NULL,
+    {"efftype::i", rOptions(Disabled, Reverb, Echo, Chorus,
+            Phaser, Alienwah, Distorsion, EQ, DynFilter)
+            rProp(parameter) rDoc("Get Effect Type"), NULL,
         [](const char *m, rtosc::RtData &d)
         {
             EffectMgr *eff  = (EffectMgr*)d.obj;
@@ -121,7 +158,14 @@ static const rtosc::Ports local_ports = {
             //Return the old data for distruction
             d.reply("/free", "sb", "EffectMgr", sizeof(EffectMgr*), &eff_);
         }},
-
+    rSubtype(Alienwah),
+    rSubtype(Chorus),
+    rSubtype(Distorsion),
+    rSubtype(DynamicFilter),
+    rSubtype(Echo),
+    rSubtype(EQ),
+    rSubtype(Phaser),
+    rSubtype(Reverb),
 };
 
 const rtosc::Ports &EffectMgr::ports = local_ports;
@@ -131,7 +175,7 @@ EffectMgr::EffectMgr(Allocator &alloc, const SYNTH_T &synth_,
     :insertion(insertion_),
       efxoutl(new float[synth_.buffersize]),
       efxoutr(new float[synth_.buffersize]),
-      filterpars(NULL),
+      filterpars(new FilterParams(time_)),
       nefx(0),
       efx(NULL),
       time(time_),
@@ -150,6 +194,7 @@ EffectMgr::EffectMgr(Allocator &alloc, const SYNTH_T &synth_,
 EffectMgr::~EffectMgr()
 {
     memory.dealloc(efx);
+    delete filterpars;
     delete [] efxoutl;
     delete [] efxoutr;
 }
@@ -171,7 +216,7 @@ void EffectMgr::changeeffectrt(int _nefx, bool avoidSmash)
     memset(efxoutr, 0, synth.bufferbytes);
     memory.dealloc(efx);
     EffectParams pars(memory, insertion, efxoutl, efxoutr, 0,
-            synth.samplerate, synth.buffersize);
+            synth.samplerate, synth.buffersize, filterpars, avoidSmash);
     try {
         switch (nefx) {
             case 1:
@@ -208,9 +253,6 @@ void EffectMgr::changeeffectrt(int _nefx, bool avoidSmash)
         return;
     }
 
-    if(efx)
-        filterpars = efx->filterpars;
-
     if(!avoidSmash)
         for(int i=0; i<128; ++i)
             settings[i] = geteffectparrt(i);
@@ -273,6 +315,10 @@ void EffectMgr::changepreset(unsigned char npreset)
 void EffectMgr::changepresetrt(unsigned char npreset, bool avoidSmash)
 {
     preset = npreset;
+    if(avoidSmash && dynamic_cast<DynamicFilter*>(efx)) {
+        efx->Ppreset = npreset;
+        return;
+    }
     if(efx)
         efx->setpreset(npreset);
     if(!avoidSmash)
@@ -409,6 +455,11 @@ void EffectMgr::paste(EffectMgr &e)
     changepresetrt(e.preset, true);
     for(int i=0;i<128;++i)
         seteffectparrt(i, e.settings[i]);
+    if(dynamic_cast<DynamicFilter*>(efx)) {
+        std::swap(filterpars, e.filterpars);
+        efx->filterpars = filterpars;
+    }
+    cleanup(); // cleanup the effect and recompute its parameters
 }
 
 void EffectMgr::add2XML(XMLwrapper& xml)
@@ -428,7 +479,8 @@ void EffectMgr::add2XML(XMLwrapper& xml)
         xml.addpar("par", par);
         xml.endbranch();
     }
-    if(filterpars) {
+    assert(filterpars);
+    if(nefx == 8) {
         xml.beginbranch("FILTER");
         filterpars->add2XML(xml);
         xml.endbranch();
@@ -454,11 +506,11 @@ void EffectMgr::getfromXML(XMLwrapper& xml)
             seteffectpar(n, xml.getpar127("par", par));
             xml.exitbranch();
         }
-        if(filterpars)
-            if(xml.enterbranch("FILTER")) {
-                filterpars->getfromXML(xml);
-                xml.exitbranch();
-            }
+        assert(filterpars);
+        if(xml.enterbranch("FILTER")) {
+            filterpars->getfromXML(xml);
+            xml.exitbranch();
+        }
         xml.exitbranch();
     }
     cleanup();
