@@ -914,6 +914,11 @@ bool Master::AudioOut(float *outr, float *outl)
     double seconds = time.time()*synth.buffersize_f/synth.samplerate_f;
     if(seconds > 10*60) {//10 minute trial
         shutup = true;
+        for(int i = 0; i < synth.buffersize; ++i) {
+            float tmp = (synth.buffersize_f - i) / synth.buffersize_f;
+            outl[i] *= 0.0f;
+            outr[i] *= 0.0f;
+        }
     }
 #endif
 
