@@ -18,6 +18,7 @@
 #include "../Synth/Resonance.h"
 #include "../Synth/OscilGen.h"
 #include "../Misc/WavFile.h"
+#include "../Misc/Time.h"
 #include <cstdio>
 
 #include <rtosc/ports.h>
@@ -39,31 +40,33 @@ static const rtosc::Ports realtime_ports =
     rRecurp(GlobalFilter, "Post Filter"),
 
     //Volume
-    rToggle(PStereo, "Stereo/Mono Mode"),
-    rParamZyn(PPanning, "Left Right Panning"),
-    rParamZyn(PVolume, "Synth Volume"),
-    rParamZyn(PAmpVelocityScaleFunction, "Amplitude Velocity Sensing function"),
+    rToggle(PStereo,    rShort("stereo"), "Stereo/Mono Mode"),
+    rParamZyn(PPanning, rShort("panning"), "Left Right Panning"),
+    rParamZyn(PVolume,  rShort("vol"), "Synth Volume"),
+    rParamZyn(PAmpVelocityScaleFunction, rShort("sense"), "Amplitude Velocity Sensing function"),
 
-    rParamZyn(Fadein_adjustment, "Adjustment for anti-pop strategy."),
+    rParamZyn(Fadein_adjustment, rShort("a.pop."), "Adjustment for anti-pop strategy."),
 
     //Punch
-    rParamZyn(PPunchStrength, "Punch Strength"),
-    rParamZyn(PPunchTime, "UNKNOWN"),
-    rParamZyn(PPunchStretch, "How Punch changes with note frequency"),
-    rParamZyn(PPunchVelocitySensing, "Punch Velocity control"),
+    rParamZyn(PPunchStrength, rShort("strength"), "Punch Strength"),
+    rParamZyn(PPunchTime,     rShort("time"),     "Length of punch"),
+    rParamZyn(PPunchStretch,  rShort("stretch"), "How Punch changes with note frequency"),
+    rParamZyn(PPunchVelocitySensing, rShort("sense"), "Punch Velocity control"),
 
     //Filter
-    rParamZyn(PFilterVelocityScale, "Filter Velocity Magnitude"),
-    rParamZyn(PFilterVelocityScaleFunction, "Filter Velocity Function Shape"),
+    rParamZyn(PFilterVelocityScale,         rShort("scale"), "Filter Velocity Magnitude"),
+    rParamZyn(PFilterVelocityScaleFunction, rShort("sense"), "Filter Velocity Function Shape"),
 
     //Freq
-    rToggle(Pfixedfreq, "Base frequency fixed frequency enable"),
-    rParamZyn(PfixedfreqET, "Equal temeperate control for fixed frequency operation"),
+    rToggle(Pfixedfreq,     rShort("fixed"),  "Base frequency fixed frequency enable"),
+    rParamZyn(PfixedfreqET, rShort("f.ET"),   "Equal temeperate control for fixed frequency operation"),
     rParamZyn(PBendAdjust,          "Pitch bend adjustment"),
-    rParamZyn(POffsetHz,          "Voice constant offset"),
-    rParamI(PDetune,        "Fine Detune"),
-    rParamI(PCoarseDetune,  "Coarse Detune"),
-    rParamZyn(PDetuneType,  "Magnitude of Detune"),
+    rParamZyn(POffsetHz,    rShort("offset"), "Voice constant offset"),
+    rParamI(PDetune,        rShort("fine"), rLinear(0, 16383), "Fine Detune"),
+    rParamI(PCoarseDetune,  rShort("coarse"), "Coarse Detune"),
+    rParamZyn(PDetuneType,  rShort("type"),
+            rOptions(L35cents, L10cents, E100cents, E1200cents),
+            "Magnitude of Detune"),
 
     {"sample#64:ifb", rProp(internal) rDoc("Nothing to see here"), 0,
         [](const char *m, rtosc::RtData &d)
@@ -85,7 +88,8 @@ static const rtosc::Ports realtime_ports =
             PADnoteParameters *obj = (PADnoteParameters *)d.obj;
             d.reply(d.loc, "f", getdetune(obj->PDetuneType, 0, obj->PDetune));
         }},
-    {"octave::c:i", rProp(parameter) rDoc("Octave note offset"), NULL,
+    {"octave::c:i", rProp(parameter) rShort("octave") rLinear(-8,7)
+        rDoc("Octave note offset"), NULL,
         [](const char *msg, RtData &d)
         {
             PADnoteParameters *obj = (PADnoteParameters *)d.obj;
@@ -99,7 +103,8 @@ static const rtosc::Ports realtime_ports =
                 obj->PCoarseDetune = k*1024 + obj->PCoarseDetune%1024;
             }
         }},
-    {"coarsedetune::c:i", rProp(parameter) rDoc("Coarse note detune"), NULL,
+    {"coarsedetune::c:i", rProp(parameter) rShort("coarse") rLinear(-64, 63)
+        rDoc("Coarse note detune"), NULL,
         [](const char *msg, RtData &d)
         {
             PADnoteParameters *obj = (PADnoteParameters *)d.obj;
@@ -136,30 +141,30 @@ static const rtosc::Ports non_realtime_ports =
     rRecurp(resonance, "Resonance"),
 
     //Harmonic Shape
-    rOption(Pmode, rMap(min, 0), rMap(max, 2), rOptions(bandwidth,discrete,continious),
+    rOption(Pmode, rMap(min, 0), rMap(max, 2), rShort("distribution"), rOptions(bandwidth,discrete,continious),
             "Harmonic Distribution Model"),
-    rOption(Php.base.type, rOptions(Gaussian, Rectanglar, Double Exponential),
+    rOption(Php.base.type, rOptions(Gaussian, Rectanglar, Double Exponential), rShort("shape"),
             "Harmonic profile shape"),
-    rParamZyn(Php.base.par1, "Harmonic shape distribution parameter"),
-    rParamZyn(Php.freqmult, "Frequency multiplier on distribution"),
-    rParamZyn(Php.modulator.par1, "Distribution modulator parameter"),
-    rParamZyn(Php.modulator.freq, "Frequency of modulator parameter"),
-    rParamZyn(Php.width, "Width of base harmonic"),
-    rOption(Php.amp.mode, rOptions(Sum, Mult, Div1, Div2),
+    rParamZyn(Php.base.par1, rShort("warp"),       "Harmonic shape distribution parameter"),
+    rParamZyn(Php.freqmult,  rShort("clone"),     "Frequency multiplier on distribution"),
+    rParamZyn(Php.modulator.par1, rShort("p1"),   "Distribution modulator parameter"),
+    rParamZyn(Php.modulator.freq, rShort("freq"), "Frequency of modulator parameter"),
+    rParamZyn(Php.width,     rShort("bandwidth"), "Width of base harmonic"),
+    rOption(Php.amp.mode,    rShort("mode"),      rOptions(Sum, Mult, Div1, Div2),
             "Amplitude harmonic multiplier type"),
 
     //Harmonic Modulation
-    rOption(Php.amp.type, rOptions(Off, Gauss, Sine, Flat),
+    rOption(Php.amp.type, rShort("mult"), rOptions(Off, Gauss, Sine, Flat),
             "Type of amplitude multipler"),
-    rParamZyn(Php.amp.par1, "Amplitude multiplier parameter"),
-    rParamZyn(Php.amp.par2, "Amplitude multiplier parameter"),
-    rToggle(Php.autoscale, "Autoscaling Harmonics"),
-    rOption(Php.onehalf,
+    rParamZyn(Php.amp.par1, rShort("p1"),   "Amplitude multiplier parameter"),
+    rParamZyn(Php.amp.par2, rShort("p2"),   "Amplitude multiplier parameter"),
+    rToggle(Php.autoscale,  rShort("auto"), "Autoscaling Harmonics"),
+    rOption(Php.onehalf, rShort("side"),
             rOptions(Full, Upper Half, Lower Half),
             "Harmonic cutoff model"),
 
     //Harmonic Bandwidth
-    rOption(Pbwscale,
+    rOption(Pbwscale, rShort("bw scale"),
             rOptions(Normal,
               EqualHz, Quater,
               Half, 75%, 150%,
@@ -171,26 +176,26 @@ static const rtosc::Ports non_realtime_ports =
             rOptions(Harmonic, ShiftU, ShiftL, PowerU, PowerL, Sine,
                 Power, Shift),
             "Harmonic Overtone shifting mode"),
-    rParamI(Phrpos.par1, rLinear(0,255), "Harmonic position parameter"),
-    rParamI(Phrpos.par2, rLinear(0,255), "Harmonic position parameter"),
-    rParamI(Phrpos.par3, rLinear(0,255), "Harmonic position parameter"),
+    rParamI(Phrpos.par1, rShort("p1"), rLinear(0,255), "Harmonic position parameter"),
+    rParamI(Phrpos.par2, rShort("p2"), rLinear(0,255), "Harmonic position parameter"),
+    rParamI(Phrpos.par3, rShort("force h."), rLinear(0,255), "Harmonic position parameter"),
 
     //Quality
-    rOption(Pquality.samplesize,
+    rOption(Pquality.samplesize, rShort("quality"),
             rOptions(16k (Tiny), 32k, 64k (Small), 128k,
               256k (Normal), 512k, 1M (Big)),
             "Size of each wavetable element"),
-    rOption(Pquality.basenote,
-            rOptions( C-2, G-2, C-3, G-3, C-4,
+    rOption(Pquality.basenote, rShort("basenote"),
+            rOptions(C-2, G-2, C-3, G-3, C-4,
                 G-4, C-5, G-5, G-6,),
             "Base note for wavetable"),
-    rOption(Pquality.smpoct,
+    rOption(Pquality.smpoct, rShort("smp/oct"),
             rOptions(0.5, 1, 2, 3, 4, 6, 12),
             "Samples per octave"),
-    rParamI(Pquality.oct, rLinear(0,7),
+    rParamI(Pquality.oct, rShort("octaves"), rLinear(0,7),
             "Number of octaves to sample (above the first sample"),
 
-    {"Pbandwidth::i", rProp(parameter) rLinear(0,1000) rDoc("Bandwith Of Harmonics"), NULL,
+    {"Pbandwidth::i", rShort("bandwidth") rProp(parameter) rLinear(0,1000) rDoc("Bandwith Of Harmonics"), NULL,
         [](const char *msg, rtosc::RtData &d) {
             PADnoteParameters *p = ((PADnoteParameters*)d.obj);
             if(rtosc_narguments(msg)) {
@@ -225,8 +230,24 @@ static const rtosc::Ports non_realtime_ports =
             float *tmp = new float[n];
             float realbw = p->getprofile(tmp, n);
             d.reply(d.loc, "b", n*sizeof(float), tmp);
-            d.reply(d.loc, "i", realbw);
+            d.reply(d.loc, "i", (int)realbw);
             delete[] tmp;}},
+    {"harmonic_profile:", rProp(non-realtime) rDoc("UI display of the harmonic profile"),
+        NULL, [](const char *m, rtosc::RtData &d) {
+            PADnoteParameters *p = ((PADnoteParameters*)d.obj);
+#define RES 512
+            char        types[RES+2] = {0};
+            rtosc_arg_t args[RES+1];
+            float tmp[RES];
+            types[0]  = 'f';
+            args[0].f = p->getprofile(tmp, RES);
+            for(int i=0; i<RES; ++i) {
+                types[i+1]  = 'f';
+                args[i+1].f = tmp[i];
+            }
+            d.replyArray(d.loc, types, args);
+#undef RES
+        }},
     {"needPrepare:", rDoc("Unimplemented Stub"),
         NULL, [](const char *, rtosc::RtData&) {}},
 };
