@@ -22,11 +22,13 @@
 #include "../Misc/Util.h"
 #include "../Misc/Allocator.h"
 #include "../Params/ADnoteParameters.h"
+#include "../Containers/ScratchString.h"
 #include "ModFilter.h"
 #include "OscilGen.h"
 #include "ADnote.h"
 
-ADnote::ADnote(ADnoteParameters *pars_, SynthParams &spars)
+ADnote::ADnote(ADnoteParameters *pars_, SynthParams &spars,
+        WatchManager *wm, const char *prefix)
     :SynthNote(spars), pars(*pars_)
 {
     memory.beginTransaction();
@@ -450,7 +452,7 @@ ADnote::ADnote(ADnoteParameters *pars_, SynthParams &spars)
         memset(tmpwave_unison[k], 0, synth.bufferbytes);
     }
 
-    initparameters();
+    initparameters(wm, prefix);
     memory.endTransaction();
 }
 
@@ -772,16 +774,17 @@ ADnote::~ADnote()
 /*
  * Init the parameters
  */
-void ADnote::initparameters()
+void ADnote::initparameters(WatchManager *wm, const char *prefix)
 {
     int tmp[NUM_VOICES];
+    ScratchString pre = prefix;
     //ADnoteParameters &pars = *partparams;
 
     // Global Parameters
     NoteGlobalPar.initparameters(pars.GlobalPar, synth,
                                  time,
                                  memory, basefreq, velocity,
-                                 stereo);
+                                 stereo, wm, prefix);
 
     NoteGlobalPar.AmpEnvelope->envout_dB(); //discard the first envelope output
     globalnewamplitude = NoteGlobalPar.Volume
@@ -816,22 +819,28 @@ void ADnote::initparameters()
 
         newamplitude[nvoice] = 1.0f;
         if(param.PAmpEnvelopeEnabled) {
-            vce.AmpEnvelope = memory.alloc<Envelope>(*param.AmpEnvelope, basefreq, synth.dt());
+            vce.AmpEnvelope = memory.alloc<Envelope>(*param.AmpEnvelope,
+                    basefreq, synth.dt(), wm,
+                    (pre+"VoicePar"+nvoice+"/AmpEnvelope/").c_str);
             vce.AmpEnvelope->envout_dB(); //discard the first envelope sample
             newamplitude[nvoice] *= vce.AmpEnvelope->envout_dB();
         }
 
         if(param.PAmpLfoEnabled) {
-            vce.AmpLfo = memory.alloc<LFO>(*param.AmpLfo, basefreq, time);
+            vce.AmpLfo = memory.alloc<LFO>(*param.AmpLfo, basefreq, time, wm,
+                    (pre+"VoicePar"+nvoice+"/AmpLfo/").c_str);
             newamplitude[nvoice] *= vce.AmpLfo->amplfoout();
         }
 
         /* Voice Frequency Parameters Init */
         if(param.PFreqEnvelopeEnabled)
-            vce.FreqEnvelope = memory.alloc<Envelope>(*param.FreqEnvelope, basefreq, synth.dt());
+            vce.FreqEnvelope = memory.alloc<Envelope>(*param.FreqEnvelope,
+                    basefreq, synth.dt(), wm,
+                    (pre+"VoicePar"+nvoice+"/FreqEnvelope/").c_str);
 
         if(param.PFreqLfoEnabled)
-            vce.FreqLfo = memory.alloc<LFO>(*param.FreqLfo, basefreq, time);
+            vce.FreqLfo = memory.alloc<LFO>(*param.FreqLfo, basefreq, time, wm,
+                    (pre+"VoicePar"+nvoice+"/FreqLfo/").c_str);
 
         /* Voice Filter Parameters Init */
         if(param.PFilterEnabled) {
@@ -843,12 +852,15 @@ void ADnote::initparameters()
 
             if(param.PFilterEnvelopeEnabled) {
                 vce.FilterEnvelope =
-                    memory.alloc<Envelope>(*param.FilterEnvelope, basefreq, synth.dt());
+                    memory.alloc<Envelope>(*param.FilterEnvelope,
+                            basefreq, synth.dt(), wm,
+                            (pre+"VoicePar"+nvoice+"/FilterEnvelope/").c_str);
                 vce.Filter->addMod(*vce.FilterEnvelope);
             }
 
             if(param.PFilterLfoEnabled) {
-                vce.FilterLfo = memory.alloc<LFO>(*param.FilterLfo, basefreq, time);
+                vce.FilterLfo = memory.alloc<LFO>(*param.FilterLfo, basefreq, time, wm,
+                        (pre+"VoicePar"+nvoice+"/FilterLfo/").c_str);
                 vce.Filter->addMod(*vce.FilterLfo);
             }
         }
@@ -892,13 +904,17 @@ void ADnote::initparameters()
         }
 
         if(param.PFMFreqEnvelopeEnabled)
-            vce.FMFreqEnvelope = memory.alloc<Envelope>(*param.FMFreqEnvelope, basefreq, synth.dt());
+            vce.FMFreqEnvelope = memory.alloc<Envelope>(*param.FMFreqEnvelope,
+                    basefreq, synth.dt(), wm,
+                    (pre+"VoicePar"+nvoice+"/FMFreqEnvelope/").c_str);
 
         FMnewamplitude[nvoice] = vce.FMVolume * ctl.fmamp.relamp;
 
         if(param.PFMAmpEnvelopeEnabled ) {
             vce.FMAmpEnvelope =
-                memory.alloc<Envelope>(*param.FMAmpEnvelope, basefreq, synth.dt());
+                memory.alloc<Envelope>(*param.FMAmpEnvelope,
+                        basefreq, synth.dt(), wm,
+                        (pre+"VoicePar"+nvoice+"/FMAmpEnvelope/").c_str);
             FMnewamplitude[nvoice] *= vce.FMAmpEnvelope->envout_dB();
         }
     }
@@ -1507,6 +1523,15 @@ inline void ADnote::ComputeVoicePinkNoise(int nvoice)
     }
 }
 
+inline void ADnote::ComputeVoiceDC(int nvoice)
+{
+    for(int k = 0; k < unison_size[nvoice]; ++k) {
+        float *tw = tmpwave_unison[k];
+        for(int i = 0; i < synth.buffersize; ++i)
+            tw[i] = 1.0f;
+    }
+}
+
 
 
 /*
@@ -1552,9 +1577,12 @@ int ADnote::noteout(float *outl, float *outr)
             case 1:
                 ComputeVoiceWhiteNoise(nvoice);
                 break;
-            default:
+            case 2:
                 ComputeVoicePinkNoise(nvoice);
                 break;
+            default:
+                ComputeVoiceDC(nvoice);
+                break;
         }
         // Voice Processing
 
@@ -1873,13 +1901,20 @@ void ADnote::Global::initparameters(const ADnoteGlobalParam &param,
                                     const AbsTime &time,
                                     class Allocator &memory,
                                     float basefreq, float velocity,
-                                    bool stereo)
+                                    bool stereo,
+                                    WatchManager *wm,
+                                    const char *prefix)
 {
-    FreqEnvelope = memory.alloc<Envelope>(*param.FreqEnvelope, basefreq, synth.dt());
-    FreqLfo      = memory.alloc<LFO>(*param.FreqLfo, basefreq, time);
+    ScratchString pre = prefix;
+    FreqEnvelope = memory.alloc<Envelope>(*param.FreqEnvelope, basefreq,
+            synth.dt(), wm, (pre+"GlobalPar/FreqEnvelope/").c_str);
+    FreqLfo      = memory.alloc<LFO>(*param.FreqLfo, basefreq, time, wm,
+                   (pre+"GlobalPar/FreqLfo/").c_str);
 
-    AmpEnvelope = memory.alloc<Envelope>(*param.AmpEnvelope, basefreq, synth.dt());
-    AmpLfo      = memory.alloc<LFO>(*param.AmpLfo, basefreq, time);
+    AmpEnvelope = memory.alloc<Envelope>(*param.AmpEnvelope, basefreq,
+            synth.dt(), wm, (pre+"GlobalPar/AmpEnvelope/").c_str);
+    AmpLfo      = memory.alloc<LFO>(*param.AmpLfo, basefreq, time, wm,
+                   (pre+"GlobalPar/AmpLfo/").c_str);
 
     Volume = 4.0f * powf(0.1f, 3.0f * (1.0f - param.PVolume / 96.0f)) //-60 dB .. 0 dB
              * VelF(velocity, param.PAmpVelocityScaleFunction);     //sensing
@@ -1887,8 +1922,10 @@ void ADnote::Global::initparameters(const ADnoteGlobalParam &param,
     Filter = memory.alloc<ModFilter>(*param.GlobalFilter, synth, time, memory, 
             stereo, basefreq);
 
-    FilterEnvelope = memory.alloc<Envelope>(*param.FilterEnvelope, basefreq, synth.dt());
-    FilterLfo      = memory.alloc<LFO>(*param.FilterLfo, basefreq, time);
+    FilterEnvelope = memory.alloc<Envelope>(*param.FilterEnvelope, basefreq,
+            synth.dt(), wm, (pre+"GlobalPar/FilterEnvelope/").c_str);
+    FilterLfo      = memory.alloc<LFO>(*param.FilterLfo, basefreq, time, wm,
+                   (pre+"GlobalPar/FilterLfo/").c_str);
 
     Filter->addMod(*FilterEnvelope);
     Filter->addMod(*FilterLfo);
