@@ -19,13 +19,15 @@
 #include <cstdio>
 #include <cmath>
 
-LFO::LFO(const LFOParams &lfopars, float basefreq, const AbsTime &t)
+LFO::LFO(const LFOParams &lfopars, float basefreq, const AbsTime &t, WatchManager *m,
+        const char *watch_prefix)
     :first_half(-1),
     delayTime(t, lfopars.Pdelay / 127.0f * 4.0f), //0..4 sec
     waveShape(lfopars.PLFOtype),
     deterministic(!lfopars.Pfreqrand),
     dt_(t.dt()),
-    lfopars_(lfopars), basefreq_(basefreq)
+    lfopars_(lfopars), basefreq_(basefreq),
+    watchOut(m, watch_prefix, "out")
 {
     int stretch = lfopars.Pstretch;
     if(stretch == 0)
@@ -166,6 +168,10 @@ float LFO::lfoout()
 
         computeNextFreqRnd();
     }
+
+    float watch_data[2] = {phase, out};
+    watchOut(watch_data, 2);
+
     return out;
 }
 
